package Dnsproxy::Cache;
use warnings;
use English '-no_match_vars';
use strict;
use Carp;
use Devel::Size qw(size total_size);

use Readonly;
use vars qw($VERSION $srvsock);

use Data::Dumper ;

our @EXPORT_OK = qw(addtocache getfromcache ipcaddtoc ipcgetfc);

$VERSION = '0.01';

Readonly my %LVL => ( INFO => 1 , WARN => 2 , ERROR => 3 , DEBUG => 4 , );
Readonly my %QTYPE => ( A => 1 , AAAA => 28 , CNAME => 5, );

my $ser = main::Data::Serializer->new() ;

sub ipcaddtoc {
my ($self,$reply ,$qid ) = @_;
$self->log ($LVL{'ERROR'}, $self->log_time . " $PROGRAM_NAME query $qid pid $PID ipcaddtoc\n" );

my $o = $ser->freeze({ pid => $PID, cmd => 'addtocache' , data => $reply, });

#my $o = freeze ({ pid => $PID, cmd => 'addtocache' , data => $reply, });

if ( not defined $srvsock ) {
	$srvsock = $self->{'server'}->{'parent_sock'};
}
		print { $srvsock } $o . "\n" or carp $ERRNO;
 		$srvsock = undef ;
return;
}


sub ipcgetfc {
my ($self,$hst,$qt,$qid) = @_;
$self->log ($LVL{'ERROR'}, $self->log_time . " $PROGRAM_NAME query $qid pid $PID ipcgetfc for $hst $qt"  . "\n" );

#my $o = freeze ({ pid => $PID, cmd => 'getfromcache' , host => $hst, qtype => $qt, });

my $o = $ser->freeze({ pid => $PID, cmd => 'getfromcache' , host => $hst, qtype => $qt,  });


if ( not defined $srvsock ) {
	$srvsock = $self->{'server'}->{'parent_sock'} ;
}

	 	print { $srvsock } $o ."\n" or carp $ERRNO;
	 	my $a = <$srvsock> ;
		if ( defined $a ) {

		chomp $a;
		my $d = eval { $ser->thaw ( $a ) } ;

		#my $d = thaw ( $a );
		
		return $d;
		}
		return;
}

sub addtocache {
my ( $self , $reply ,$qid ) = @_;
$self->log ($LVL{'ERROR'}, $self->log_time . " $PROGRAM_NAME query $qid pid $PID add reply to cache\n" );
my $ti = time;

my $answer_hash = {};

my @answer = @ { $reply->{'answer'} };
my @question = @ { $reply->{'question'} } ;


my ($qkey,$rdatastr,$ttl,$expire,$type,$qtype );


for my $rr ( @question ) {
	$qkey = $rr->{'qname'}->{'name'} ;
	$qtype = $rr->{'qtype'} ;

}

# delete cache item if item-expire < time
 if ( $main::dnscache{'meta'}{ $qtype }->{$qkey}->{'expire'} ) {

	if ( $main::dnscache{'meta'}{ $qtype }->{$qkey}->{'expire'} < $ti ) {
		delete $main::dnscache{'meta'}{ $qtype }->{$qkey};
		if ( $main::dnscache{'data'}{ $qtype }->{$qkey} ) {
			delete $main::dnscache{'data'}{ $qtype }->{$qkey};
		}
	}

	if ( $main::dnscache{'meta'}{ $qtype }->{$qkey}->{'expire'} > $ti ) {
		return ;
	}

}

for my $rr ( @answer ) {
	$ttl = $rr->{'ttl'};
	$rdatastr = $rr->{'owner'}->{'name'};
	$expire = $ttl + $ti;
	$type = $rr->{'type'} ;
		$main::dnscache{'meta'}{ $type }->{$rdatastr} = {
		expire => $expire ,
		ttl => $ttl,
		type => $type,
		class => $rr->{'class'},
	}
}

$main::dnscache{'data'}{$qtype}->{$qkey} = {
	answer => $reply->{'answer'},
	authority => $reply->{'authority'},
	additional => $reply->{'additional'},
};

#$self->dumpcache ( $qid );
return;
}

sub dumpcache {
my ($self,$qid ) = @_;
$self->log ($LVL{'ERROR'}, $self->log_time . " $PROGRAM_NAME query $qid pid $PID cache size "  . size \%main::dnscache . "\n" );
$self->log ($LVL{'DEBUG'}, $self->log_time . " $PROGRAM_NAME query $qid pid $PID dump Cache\n" );
print {*STDOUT} main::Dumper %main::dnscache or carp $ERRNO;

return;
}


sub getfromcache {
my ($self,$hst,$qt,$qid ) = @_;
$self->log ($LVL{'DEBUG'}, $self->log_time . " $PROGRAM_NAME query/PID $qid pid $PID getfromcache $hst $qt\n" );
my $ti = time;
my $type = $QTYPE{$qt};

if ( defined $main::dnscache{'meta'}{ $type }->{$hst}->{'expire'} ) {

		if ( $main::dnscache{'meta'}{ $type }->{$hst}->{'expire'} > $ti ) {
			$self->log ($LVL{'DEBUG'}, $self->log_time . " $PROGRAM_NAME query $qid pid $PID object $hst is in cache\n" );
			$self->log ($LVL{'DEBUG'}, $self->log_time . " $PROGRAM_NAME query $qid pid $PID cache reply type $qt for $hst!\n" );
			my $reply = $self->empty_reply;
			$reply->push ( answer => @ { $main::dnscache{'data'}{ $type }->{$hst}->{'answer'} } );
			#$reply->push ( authority => @ { $main::dnscache{'data'}{ $type }->{$hst}->{'authority'} } );
			#$reply->push ( additional => @ { $main::dnscache{'data'}{ $type }->{$hst}->{'additional'} } );
			return $reply;
		}

}
return ;
}



1;
__END__