package Dnsproxy::Dns;
use warnings;
use English '-no_match_vars';
use strict;
use Carp;
use Readonly ;

use Net::DNS::Packet;
use Net::DNS::RR;
use Net::DNS::Question ;

use Data::Dumper;

Readonly my %LVL => ( INFO => 1 , WARN => 2 , ERROR => 3 , DEBUG => 4 , TTL => 172_800 , );

use vars qw($VERSION);

$VERSION = '0.01';

use Exporter qw(import);
use base qw(Exporter);
our @EXPORT_OK = qw(empty_reply blackhost_reply dnsquery);

sub dnsquery {
my ($self,$buf,$peerhost,$dbg) =  @_;

my $query = Net::DNS::Packet->new ( \${$buf},$dbg );
my $reply ;

	if ( not $query ) {
		$self->log ($LVL{'DEBUG'} , $self->log_time . " $PROGRAM_NAME Error decoding query packet: $EVAL_ERROR\n");
		$reply = empty_reply ();
		$reply->header->rcode('FORMERR');
		undef $query; # force FORMERR reply
	}

	if ( $query->header->qr() ) {
		$self->log ($LVL{'ERROR'}, $self->log_time . " $PROGRAM_NAME invalid packet (qr set), dropping\n" );
		return;
	}

	if ( not defined $reply ) {
		$reply = $self->make_reply( $query, $peerhost ) || return;
	}

	my $max_len ;
	if  ( $query &&  $self->get_property('truncate') ) {
		$max_len = $query->edns->size;
	}
	return ( $reply , $max_len );
}
sub empty_reply {
		my $empty = Net::DNS::Packet->new(); # create empty reply packet
		my $reply = $empty->reply();
		$reply->answerfrom('127.0.0.1');
		my $h = $reply->header;
		$h->rcode('NOERROR');
		return $reply;
}

sub blackhost_reply {
my ( $qname ,$qtype , $qid )  = @_;
my $empty = Net::DNS::Packet->new(); # create empty reply packet

my $lo4 = Net::DNS::RR->new( owner => $qname,
                             type => 'A',
                             address => '127.0.0.1',
                             ttl => $LVL{'TTL'} ,
                        );

my $qlo4 = Net::DNS::Question->new($qname, 'A', 'IN' );
$qlo4->{'qname'}->{'name'} = $qname;


my $lo6 = Net::DNS::RR->new( type => 'AAAA',
										owner => $qname,
                             address => '::1',
                             ttl => $LVL{'TTL'} ,
                        );


my $qlo6 = Net::DNS::Question->new($qname, 'AAAA', 'IN' );
$qlo6->{'qname'}->{'name'} = $qname;

my $reply = $empty->reply();
$reply->answerfrom('127.0.0.1');

my $h = $reply->header;
$h->rcode('NOERROR');
$h->opcode('QUERY');
$h->qr('1');
$h->rd('1');

if ( $qtype eq 'AAAA') {
	$reply->push ( answer => $lo6 );
	$reply->push ( question => $qlo6 );
} else {
	$reply->push ( answer => $lo4 );
	$reply->push ( question => $qlo4 );
}

return $reply;
}
1;
__END__