package Dnsproxy::Dnsproxy;
use strict;
use warnings;
use English '-no_match_vars';
use Carp;
use Readonly ;
use feature qw(switch);
no warnings 'experimental::smartmatch';

use lib 'lib/';

use Dnsproxy::Adblock qw(matchhost);
use Dnsproxy::Resolver qw(resolver);
use Dnsproxy::Dns qw(empty_reply blackhost_reply dnsquery);

Readonly my %LVL => ( INFO => 1 , WARN => 2 , ERROR => 3 , DEBUG => 4 , );

#use Data::Dumper ;

use vars qw($VERSION);

use Exporter qw(import);
use base qw(Exporter);
our @EXPORT_OK = qw(blacktaghandler replyhandler make_reply empty_reply blackhost_reply dnsquery resolver matchhost);

$VERSION = '0.01';


sub blacktaghandler {
my ($self,$args) = @_ ;
my ( $reply,$rcode , @ans , @auth , @add );
$self->log ($LVL{'DEBUG'}, $self->log_time . " $PROGRAM_NAME query $args->{'id'} : blacklist $args->{'qname'}\n");

	# caching
	$self->log ($LVL{'DEBUG'} ,$self->log_time . " $PROGRAM_NAME query $args->{'id'} : blacklist call getfromcache $args->{'qname'} $args->{'qtype'}\n");
		
		if ( defined $self->{'server'}->{'parent_sock'} ) {
			$reply = $self->ipcgetfc  ( $args->{'qname'}, $args->{'qtype'} , $args->{'id'}  );
		} else {
			$reply = $self->getfromcache ( $args->{'qname'}, $args->{'qtype'} , $args->{'id'} );
		}

	if ( not defined $reply ) {
   	$reply = blackhost_reply ( $args->{'qname'} , $args->{'qtype'} , $args->{'id'} ) ;
	}

	#print Dumper $reply ;

		$rcode = $reply->header->rcode;
		#$reply->push ( answer => $reply );

		@ans   = $reply->answer;
		@auth  = $reply->authority;
		@add   = $reply->additional;

	$self->log ($LVL{'DEBUG'},$self->log_time . " $PROGRAM_NAME reply to $args->{'peerhost'} : blacklist $args->{'id'} $args->{'qname'} rcode $rcode\n");

	return ($rcode, \@ans, \@auth, \@add);
}

sub replyhandler {
	my ($self,$args) = @_;

   $self->log ($LVL{'DEBUG'}, $self->log_time . " $PROGRAM_NAME query $args->{'id'} pid $PID reply handler $args->{'qname'}");
   my (@ans, @auth, @add);
   my $rcode = 'NXDOMAIN';
  	my $query ;

	# caching
	$self->log ($LVL{'DEBUG'} ,$self->log_time . " $PROGRAM_NAME query $args->{'id'} pid $PID call getfromcache $args->{'qname'} $args->{'qtype'}\n");

	if ( defined $self->{'server'}->{'parent_sock'} ) {
		$query = $self->ipcgetfc  ( $args->{'qname'}, $args->{'qtype'} , $args->{'id'}  );
	} else {
		$query = $self->getfromcache ( $args->{'qname'}, $args->{'qtype'} , $args->{'id'} );
	}

	#print main::Dumper $query;
	
	# wenn ueber ipcgetfc nichts findet

	#if ( ( defined $query->{'data'} ) and ( $query->{'data'} eq 'NOTFOUNDINCACHE' ) ) {
	#	$self->log (DEBUG ,$self->log_time . " $PROGRAM_NAME query $args->{'id'} pid $PID call dns resolver $args->{'qname'} $args->{'qtype'} \n");
	#	$query = $self->resolver( $args->{'qname'}, $args->{'qtype'} );		
	#}

	if ( not defined $query ) {
		$self->log ($LVL{'DEBUG'} ,$self->log_time . " $PROGRAM_NAME query $args->{'id'} pid $PID call dns resolver $args->{'qname'} $args->{'qtype'} \n");
		$query = $self->resolver( $args->{'qname'}, $args->{'qtype'} );
	}

	#print main::Dumper $query;

 	if ( $query ) {
 		$rcode = $query->header->rcode;
		@ans   = $query->answer;
		@auth  = $query->authority;
		@add   = $query->additional;
		$self->log ($LVL{'DEBUG'},$self->log_time . " $PROGRAM_NAME query $args->{'id'} pid $PID reply to $args->{'peerhost'} query $args->{'qname'} rcode $rcode\n");

		#print main::Dumper @ans;

		return ($rcode, \@ans, \@auth, \@add);
    }
return ($rcode, \@ans, \@auth, \@add);
}


sub make_reply {
	my ( $self, $query, $peerhost ) = @_;

	my $reply  = $query->reply();
	my $header = $reply->header;
	my $headermask;
	my $optionmask;

	my $opcode  = $query->header->opcode;
	my $qdcount = $query->header->qdcount;

	my ($qr,$qname,$qtype,$qclass,$id,$rcode, $ans, $auth, $add);

	if ( not $qdcount ) {
		$header->rcode('NOERROR');

	} elsif ( $qdcount > 1 ) {
		$self->log ($LVL{'ERROR'}, $self->log_time . " $PROGRAM_NAME qdcount $qdcount unsupported\n");
		$header->rcode('FORMERR');

	} else {
			($qr)     = $query->question ;
			$qname  = $qr->qname;
			$qtype  = $qr->qtype;
			$qclass = $qr->qclass;
			$id = $query->header->id;
			$self->log ($LVL{'DEBUG'} ,$self->log_time . " $PROGRAM_NAME query $id pid $PID query $qname $qclass qclass qtpye $qtype\n");
	}

	#### adblock 
	my $t = matchhost ( $qname );

	if ( not $#{$t} < 0 ) {
		$opcode = 'BLACKTAGQUERY';
		$self->log ($LVL{'DEBUG'},$self->log_time . " $PROGRAM_NAME query $id : blacklist match hostname $qname\n");
	}

	my $arglist = ({ qname => $qname,
							qclass => $qclass,
							qtype => $qtype,
							peerhost => $peerhost,
							query => $query,
							id => $id,
						});

		given ( $opcode ) {
			when ( $_ eq 'QUERY' ) {
				( $rcode, $ans, $auth, $add, $headermask, $optionmask ) =
					$self->replyhandler ($arglist);
			}
			when ( $_ eq 'BLACKTAGQUERY' ) {
				( $rcode, $ans, $auth, $add, $headermask, $optionmask ) =
					$self->blacktaghandler ($arglist);
			}

			#when ( $_ eq 'NOTIFY' ) {
			#	if ( ref $self->{notifyhandler} eq 'CODE' ) {
			#		( $rcode, $ans, $auth, $add, $headermask, $optionmask ) =
			#			$self->notifyhandler ($arglist);
			#	} else {
			#		$rcode = 'NOTIMP';
			#	}
			#}
			#when ( $_ eq 'UPDATE' ) {
			#	if ( ref $self->{updatehandler} eq 'CODE' ) {
			#		( $rcode, $ans, $auth, $add, $headermask, $optionmask ) =
			#			$self->updatehandler ($arglist);
			#	} else {
			#		$rcode = 'NOTIMP';
			#	}
			#}
			default {
				$self->log  ($LVL{'ERROR'} , $self->log_time . " $PROGRAM_NAME opcode $opcode unsupported\n");
				$rcode = 'FORMERR';
			}
		}


		if ( not defined $rcode ) {
			$self->log ($LVL{'WARN'}, $self->log_time . " $PROGRAM_NAME remaining silent\n");
			return ;
		}


		$header->rcode($rcode);
		if ( $ans ) {
			$reply->{answer}     =  [@{$ans}];
		}
		#if ( $auth ) {
		#	$reply->{authority}  = [@{$auth}];
		#}
		#if ( $add ) {
 		#	$reply->{additional} = [@{$add}];
 		#}


	while ( my ( $key, $value ) = each %{$headermask || {}} ) {
		$header->$key($value);
	}

	while ( my ( $option, $value ) = each %{$optionmask || {}} ) {
		$reply->edns->option( $option, $value );
	}

	if  ( ( $self->get_property('log_level') >= $LVL{'DEBUG'} ) && ( $headermask || $optionmask ) ) {
		$header->print;
	}
# print main::Dumper $reply;  
	if ( defined $self->{'server'}->{'parent_sock'} ) {
		$self->ipcaddtoc ($reply,$id);
	} else {
		$self->addtocache ($reply,$id);
	}

	return $reply;
}

1;
__END__
