package Dnsproxy::Server;
use strict;
use warnings;
use English '-no_match_vars';
use Data::Serializer;
#use Storable qw(freeze thaw);


use Carp;
use feature qw(switch);
no warnings 'experimental::smartmatch';
use lib 'lib/';
use vars qw($VERSION %dnscache);
use Try::Tiny;
use Readonly ;
use base qw(Net::Server::PreFork Dnsproxy::Cache);
#use base qw(Net::Server Dnsproxy::Cache);


use Data::Dumper ;


Readonly my $PACKETSZ => 512;
Readonly my %LVL => ( INFO => 1 , WARN => 2 , ERROR => 3 , DEBUG => 4 , );

use Dnsproxy::Dnsproxy qw(blacktaghandler replyhandler make_reply empty_reply blackhost_reply dnsquery resolver);

$VERSION = '0.01';

my $ser = Data::Serializer->new();

sub child_is_talking_hook {
my ( $self,$data ) = @_ ;
my $a = <$data>;
if ( not $a ) {  return ; }
chomp $a;

my $o = $ser->thaw ( $a );

#my $o = thaw ( $a ) ;
#print Dumper $o;

given ( $o->{'cmd'} ) {
	$self->log ($LVL{'DEBUG'} , $self->log_time . " $PROGRAM_NAME pid $PID child is talking ...\n" );

	when ( $_ eq 'getfromcache' ) {
		$self->log ($LVL{'DEBUG'} , $self->log_time . " $PROGRAM_NAME pid $PID child get data from cache ...\n" );
		my $t = $self->getfromcache ( $o->{'host'}, $o->{'qtype'} ,$PID ) ;

	 	my $aa = $ser->freeze ( $t ) or carp $ERRNO;
#	  	my $aa = freeze ( $t ) or carp $ERRNO;
	 	
		print {$data}  $aa . "\n" or carp $ERRNO;
	}
	when ( $_ eq 'addtocache' ) {
		$self->log ($LVL{'DEBUG'} , $self->log_time . " $PROGRAM_NAME pid $PID child add data to cache ...\n" );
		$self->addtocache ($o->{'data'},$PID );
	}

$self->log ($LVL{'DEBUG'} , $self->log_time . " $PROGRAM_NAME pid $PID child talking done ...\n" );
}


#$data->autoflush(1);

return;
}

sub process_request {
my ($self,$sock) = @_;
my $cln = $self->get_property('client');

	$self->{'server'}->{'udp_args'}->{'udp_recv_len'} = $PACKETSZ;
	my $buf = $self->get_property( 'udp_data' );

#	my ( $peerhost, $peerport, $sockhost , $sockport ) = (
	my ( $peerhost, $peerport ) = (
		$self->get_property( 'peeraddr' ),
		$self->get_property( 'peerport' ),
	);

	if ( not defined ( $peerhost and $peerport ) ) {
			$self->log ($LVL{'DEBUG'} , $self->log_time . " $PROGRAM_NAME the Peer and the sock host appear to be undefined: bailing out of handling the UDP connection\n" );
			return;
	}

	my ( $reply,$max_len ) = $self->dnsquery (\$buf , $peerhost, $self->get_property( 'debug'), ) ;

	if (  $self->get_property('log_level') >= $LVL{'DEBUG'}  ) {
		if ( $max_len ) {
			$self->log ($LVL{'DEBUG'}, $self->log_time . " $PROGRAM_NAME Maximum UDP size advertised by $peerhost#$peerport: $max_len\n" );
		}
		$cln->send ( $reply->data($max_len) ) or $self->log ($LVL{'DEBUG'}, $self->log_time . " $PROGRAM_NAME failed: $ERRNO\n") ;

	} else {
		$cln->send( $reply->data($max_len) );
	}

 	$buf = undef;
	return ;
}
#sub post_process_request_hook {
#my ($self,$sock) = @_;
#$self->dumpcache;
#return;
#}
sub options {
    my $self     = shift;
    my $prop     = $self->{'server'};
    my $template = shift;

    # setup options in the parent classes
    $self->SUPER::options($template);

	 $prop->{'child_communication'} ||= 'true';
    $template->{'child_communication'} = \ $prop->{'child_communication'};

	 $prop->{'debug'} ||= undef;
    $template->{'debug'} = \ $prop->{'debug'};

    $prop->{'whitelistfile'} ||= undef;
    $template->{'whitelistfile'} = \ $prop->{'whitelistfile'};

    $prop->{'blackholeip4'} ||= undef;
    $template->{'blackholeip4'} = \ $prop->{'blackholeip4'};

    $prop->{'blackholeip6'} ||= undef;
    $template->{'blackholeip6'} = \ $prop->{'blackholeip6'};

     $prop->{'truncate'} ||= undef;
     $template->{'truncate'} = \ $prop->{'truncate'};

     $prop->{'resolverport'} ||= undef;
     $template->{'resolverport'} = \ $prop->{'resolverport'};

     # add a multi value option
    $prop->{'nameservers'} ||= [];
    $template->{'nameservers'} = $prop->{'nameservers'};

    $prop->{'searchlist'} ||= [];
    $template->{'searchlist'} = $prop->{'searchlist'};

    $prop->{'blacklistfile'} ||= [];
    $template->{'blacklistfile'} = $prop->{'blacklistfile'};

    $prop->{'min_servers'} ||= undef ;
    $template->{'min_servers'} = $prop->{'min_servers'};

    $prop->{'min_spare_servers'} ||= undef ;
    $template->{'min_spare_servers'} = $prop->{'min_spare_servers'};

    $prop->{'max_servers'} ||= undef ;
    $template->{'max_servers'} = $prop->{'max_servers'};


   return ;
}

sub post_configure_hook {
my $self = shift;

$self->log ($LVL{'DEBUG'},$self->log_time . " $PROGRAM_NAME init $PROGRAM_NAME Daemon\n");


	if ( not defined $self->get_property('resolverport') ) {
		$self->set_property('resolverport' => '53' );
	}
	if ( not defined $self->get_property('blackholeip4') ) {
		$self->set_property('blackholeip4' => '127.0.0.1' );
	}
	if ( not defined $self->get_property('blackholeip6') ) {
		$self->set_property('blackholeip6' => '::1' );
	}
	if ( not defined $self->get_property('min_servers') ) {
		$self->set_property('min_servers' => '1' );
	}
	if ( not defined $self->get_property('min_spare_servers') ) {
		$self->set_property('min_spare_servers' => '2' );
	}
	if ( not defined $self->get_property('max_servers') ) {
		$self->set_property('max_servers' => '3' );
	}
	# init adblock
	$self->log ($LVL{'DEBUG'},$self->log_time . " $PROGRAM_NAME init black and whitelist files\n");
	try {
		Dnsproxy::Adblock::initadblock ({
					blacklist => $self->get_property('blacklistfile'),
					whitelist => $self->get_property('whitelistfile'),
		}) ;
	} catch { croak $_ ; };

	# init resolver 
	try {
		Dnsproxy::Resolver::initresolver ({
			nameservers => $self->get_property('nameservers'),
			serchlist => $self->get_property('searchlist'),
			log_level => $self->get_property('log_level'),
			proto => $self->get_property('proto'),
			port => $self->get_property('resolverport'),
		}) ;
	} catch { croak $_ ; } ;

	return ;
}

1;
__END__