package Dnsproxy::Config;
use warnings;
use feature qw(switch);
no warnings "experimental::smartmatch";
#use experimental 'switch';
use English '-no_match_vars';
use strict;
use Carp;

use Config::ApacheFormat;
use Cwd qw(cwd);
use Env qw(HOME);

use Data::Dumper;

use Exporter qw(import);
use base qw(Exporter);
our @EXPORT_OK = qw(readconfigfile);


use vars qw($VERSION);

$VERSION = '0.01';


my $config = Config::ApacheFormat->new(
	inheritance_support => 0,
   include_support     => 0,
   fix_booleans        => 1,
);


sub new {
	my ($class,$args) = @_;
   my $self;
	if ( defined $args->{'verbose'} ) {
   	$self->{'verbose'} = $args->{'verbose'};
   }
	if ( defined $args->{'nodmn'} ) {
   	$self->{'nodaemon'} = $args->{'nodmn'};
   }
   bless $self, $class;
   return $self;
}

##############################

sub readconfigfile {
my ( $self,$file ) = @_;

    my @cpath = qw(/etc/dnsproxy.conf);
    push @cpath, $HOME . '/.dnsproxy.conf';
    push @cpath, cwd . '/dnsproxy.conf';

    if ( defined $file ) { push @cpath,$file ; }

    for my $f ( @cpath ) {
		if ( -f $f ) {
		$self->{'conf_file'} = $f;
		last;
		}
    }
	if ( not defined $self->{'conf_file'} ) { croak 'config file not found' ; }
	$config->read($self->{'conf_file'});
	
	my %dopts = (
		conf_file => '/etc/dnsproxy.conf',
		logfile => 'dnsproxy.log',
		pidfile => 'dnsproxy.pid',
		host => '*',
		debug => '0',
		user => 'nobody',
		group => 'daemon',
		port => '53',
		listen => '127.0.0.1',
		home => cwd,
		verbose => '0',
		truncate => '0',
		ipv	=> '4',
		proto => 'unix', );


	 my $block = $config->block('daemon');
		
	for my $k ( keys %dopts ) {
		 if ( not defined $self->{'server'}->{$k} ) {
			$self->{$k} = $block->get($k) || $dopts{$k} ;
		 }
	}
	
	
   
    if ( ($self->{'verbose'} < 0 ) or ( $self->{'verbose'} > 2) ) {
        my $err = 'verbosity must be an integer between 0 and 2';
        Dnsproxy::Dnsproxy::out('ERROR', $err);
        carp $err;
    }

    $self->{'uid'}     = scalar getpwnam $self->{'user'}  || scalar getpwnam 'nobody' ;
    $self->{'gid'}     = scalar getgrnam $self->{'group'} || scalar getgrnam 'daemon' ;

   my $rname ;
   for my $r ($config->get('resolver')) {
        $rname = $r->[1];
        $self->{'_resolvers'}->{$rname} = {};
        $block = undef;
        $block = $config->block('resolver' => $rname);
        $self->{'_resolvers'}->{$rname}->{'trigger'} = $block->get('trigger');
        $self->{'_resolvers'}->{$rname}->{'tcp'} = $block->get('tcp');
        if ( $block->get('tcp') !~ /^(?:true|[0])$/smx ) {
        	$self->{'tcp'}         = $block->get('tcp');
        }

        if ( (  $block->get('default') ) and ( $self->{'verbose'} == 2 ) ) {
            Dnsproxy::Dnsproxy::out('DEBUG', "resolver $rname is default\n");
            $self->{'_resolvers'}->{$rname}->{'default'} = 1;
            $self->{'default_resolver'} = $rname;
        }

	     my @ns = $block->get('nameserver');
        my @searchlist  = $block->get('searchlist');

        $self->{'nameservers'} = \@ns;
		  $self->{'searchlist'} = \@searchlist;

        $self->{'_resolvers'}->{$rname}->{'tcptimeout'}          = $block->get('tcptimeout')  || '5' ;
        $self->{'_resolvers'}->{$rname}->{'udptimeout'}          = $block->get('udptimeout')  || '5' ;

	}
		  $block = $config->block('blacklist');
        $self->{'blacklist'}          = $block->get('file')  ;
        $self->{'blackholeip4'}          = $block->get('blackholeip4')  ;
        $self->{'blackholeip6'}          = $block->get('blackholeip6')  ;

#print Dumper $self;
#exit;
return ;
}


1;
__END__
