package Dnsproxy::Dnsproxy;
use strict;
use warnings;
use English '-no_match_vars';
use Carp;

use feature qw(switch);
no warnings 'experimental::smartmatch';

use lib 'lib/';

use Dnsproxy::Adblock qw(matchhost);
use Dnsproxy::Resolver qw(resolver mresolver);
use Dnsproxy::Dns qw(empty_reply blackhost_reply dnsquery);


use constant PACKETSZ => 512;
use constant { INFO => 1 , WARN => 2 , ERROR => 3 , DEBUG => 4 , };


use vars qw($VERSION);
#use base qw(Net::Server);

use Exporter qw(import);
use base qw(Exporter);
our @EXPORT_OK = qw(blacktaghandler replyhandler make_reply empty_reply blackhost_reply dnsquery resolver matchhost);

$VERSION = '0.01';


sub blacktaghandler {
my ($self,$args) = @_ ;

$self->log (DEBUG, $self->log_time . " $PROGRAM_NAME query $args->{'id'} : query $args->{'qname'} :  blacktag handler\n");
my $reply = blackhost_reply ( $args->{'qname'} , $args->{'qtype'} ) ;

	my $rcode = $reply->header->rcode;
	my @ans   = $reply->answer;
	my @auth  = $reply->authority;
	my @add   = $reply->additional;
	$self->log (INFO,$self->log_time . "$PROGRAM_NAME query $args->{'id'} : reply to $args->{'peerhost'} : query rcode $rcode\n");

	return ($rcode, \@ans, \@auth, \@add);
}
sub replyhandler {
	my ($self,$args) = @_;
	
   $self->log (DEBUG, $self->log_time . " $PROGRAM_NAME query $args->{'id'} : query $args->{'qname'} : reply handler");
   my (@ans, @auth, @add);
   my $rcode = 'NXDOMAIN';
   
	
  	my $qu = $self->resolver( $args->{'qname'}, $args->{'qtype'} );
  	
 	if ( $qu ) {
 		$rcode = $qu->header->rcode;
		@ans   = $qu->answer;
		@auth  = $qu->authority;
		@add   = $qu->additional;
		$self->log (INFO,$self->log_time . " $PROGRAM_NAME query $args->{'id'} : reply to $args->{'peerhost'} : query rcode $rcode\n");
		return ($rcode, \@ans, \@auth, \@add);
    }
return ($rcode, \@ans, \@auth, \@add);
}

sub o_process_request {
my ($self,$sock) = @_;
my $cln = $self->get_property('client');

	$self->{'server'}->{'udp_args'}->{'udp_recv_len'} = PACKETSZ;
	my $buf = $self->get_property( 'udp_data' );
	

	my ( $peerhost, $peerport, $sockhost , $sockport ) = (
		$self->get_property( 'peeraddr' ),
		$self->get_property( 'peerport' ),
		$self->get_property( 'sockaddr' ),
		$self->get_property( 'sockport' ),
	);

	if ( not defined ( $peerhost and $peerport ) ) {
			$self->log (DEBUG , $self->log_time . " $PROGRAM_NAME the Peer and the sock host appear to be undefined: bailing out of handling the UDP connection\n" );
			return;
	}

	my $query = dnsquery (\$buf , $self->get_property( 'debug' ) ) ;

	my $reply;

	if ( not $query ) {
		$self->log (DEBUG , $self->log_time . " $PROGRAM_NAME Error decoding query packet: $EVAL_ERROR\n");
		$reply = empty_reply ();
		$reply->header->rcode('FORMERR');
		undef $query; # force FORMERR reply
	}

	if ( $query->header->qr() ) {
		$self->log (ERROR, $self->log_time . " $PROGRAM_NAME invalid packet (qr set), dropping\n" );
		return;
	}

	if ( not defined $reply ) {
		$reply = $self->make_reply( $query, $peerhost ) || return;
	}

	my $max_len ;
	if  ( $query &&  $self->get_property('truncate') ) {
		$max_len = $query->edns->size;
	}

	if (  $self->get_property('log_level') >= DEBUG  ) {
		if ( $max_len ) {
			$self->log (DEBUG, $self->log_time . " $PROGRAM_NAME Maximum UDP size advertised by $peerhost#$peerport: $max_len\n" );
		}
		$cln->send ( $reply->data($max_len) ) or $self->log (DEBUG, $self->log_time . " $PROGRAM_NAME failed: $ERRNO ") ;

	} else {
		$cln->send( $reply->data($max_len) );
	}
	$buf = undef;
	return ;
}

sub make_reply {
	my ( $self, $query, $peerhost ) = @_;

 #my @ca = caller 1;
 #print main::Dumper $ca[3];

	my $reply  = $query->reply();
	my $header = $reply->header;
	my $headermask;
	my $optionmask;

	my $opcode  = $query->header->opcode;
	my $qdcount = $query->header->qdcount;

	my ($qr,$qname,$qtype,$qclass,$id,$rcode, $ans, $auth, $add);

	if ( not $qdcount ) {
		$header->rcode('NOERROR');

	} elsif ( $qdcount > 1 ) {
		$self->log (ERROR, $self->log_time . " $PROGRAM_NAME qdcount $qdcount unsupported\n");
		$header->rcode('FORMERR');

	} else {
			($qr)     = $query->question ;
			$qname  = $qr->qname;
			$qtype  = $qr->qtype;
			$qclass = $qr->qclass;
			$id = $query->header->id;
			$self->log (DEBUG ,$self->log_time . " $PROGRAM_NAME query $id : $qname $qclass $qtype - \n");
	}
	#### adblock 
	my $t = matchhost ( $qname );
	if ( not $#{$t} < 0 ) {
		$opcode = 'BLACKTAGQUERY';
		$self->log (DEBUG,$self->log_time . " $PROGRAM_NAME query $id : Adblock match hostname $qname\n");
	}

		#my ( $rcode, $ans, $auth, $add );
		my $arglist = ({ qname => $qname,
								qclass => $qclass,
								qtype => $qtype,
								peerhost => $peerhost,
								query => $query,
								id => $id,
								});


		given ( $opcode ) {
			when ( $_ eq 'QUERY' ) {
				( $rcode, $ans, $auth, $add, $headermask, $optionmask ) =
					$self->replyhandler ($arglist);
			}
			when ( $_ eq 'BLACKTAGQUERY' ) {
				#if ( ref $self->{blacktaghandler} eq 'CODE' ) {
					( $rcode, $ans, $auth, $add, $headermask, $optionmask ) =
					$self->blacktaghandler ($arglist);
				#} else {
				#	$rcode = 'NOTIMP';
				#;}
			}
			when ( $_ eq 'NOTIFY' ) {
				if ( ref $self->{notifyhandler} eq 'CODE' ) {
					( $rcode, $ans, $auth, $add, $headermask, $optionmask ) =
						$self->notifyhandler ($arglist);
				} else {
					$rcode = 'NOTIMP';
				}
			}
			when ( $_ eq 'UPDATE' ) {
				if ( ref $self->{updatehandler} eq 'CODE' ) {
					( $rcode, $ans, $auth, $add, $headermask, $optionmask ) =
						$self->updatehandler ($arglist);
				} else {
					$rcode = 'NOTIMP';
				}
			}
			default {
				$self->log  (ERROR , $self->log_time . " $PROGRAM_NAME opcode $opcode unsupported\n");
				$rcode = 'FORMERR';
			}
		}

		if ( not defined $rcode ) {
			$self->log (WARN, $self->log_time . " $PROGRAM_NAME remaining silent\n");
			return ;
		}

		$header->rcode($rcode);
		if ( $ans ) {
			$reply->{answer}     =  [@{$ans}];
		}
		if ( $auth ) {
			$reply->{authority}  = [@{$auth}];
		}
		if ( $add ) {
 			$reply->{additional} = [@{$add}];
 		}


	while ( my ( $key, $value ) = each %{$headermask || {}} ) {
		$header->$key($value);
	}

	while ( my ( $option, $value ) = each %{$optionmask || {}} ) {
		$reply->edns->option( $option, $value );
	}

	if  ( ( $self->get_property('log_level') >= DEBUG ) && ( $headermask || $optionmask ) ) {
		$header->print;
	}
	#out ('INFO' , "peerhost: $peerhost query: $qname\n"  );

	return $reply;
}

sub o_options {
    my $self     = shift;
    my $prop     = $self->{'server'};
    my $template = shift;

    # setup options in the parent classes
    $self->SUPER::options($template);

	 $prop->{'debug'} ||= undef;
    $template->{'debug'} = \ $prop->{'debug'};

    $prop->{'whitelistfile'} ||= undef;
    $template->{'whitelistfile'} = \ $prop->{'whitelistfile'};

    $prop->{'blackholeip4'} ||= undef;
    $template->{'blackholeip4'} = \ $prop->{'blackholeip4'};

    $prop->{'blackholeip6'} ||= undef;
    $template->{'blackholeip6'} = \ $prop->{'blackholeip6'};

     $prop->{'truncate'} ||= undef;
     $template->{'truncate'} = \ $prop->{'truncate'};

     $prop->{'resolverport'} ||= undef;
     $template->{'resolverport'} = \ $prop->{'resolverport'};

     # add a multi value option
    $prop->{'nameservers'} ||= [];
    $template->{'nameservers'} = $prop->{'nameservers'};

    $prop->{'searchlist'} ||= [];
    $template->{'searchlist'} = $prop->{'searchlist'};

    $prop->{'blacklistfile'} ||= [];
    $template->{'blacklistfile'} = $prop->{'blacklistfile'};

   return ;
}

sub o_post_configure_hook {
my $self = shift;
$self->log ( DEBUG,$self->log_time . " $PROGRAM_NAME init $PROGRAM_NAME Daemon\n");

	if ( not defined $self->get_property('resolverport') ) {
		$self->set_property('resolverport' => '53' );
	}
	
	if ( not defined $self->get_property('blackholeip4') ) {
		$self->set_property('blackholeip4' => '127.0.0.1' );
	}
		if ( not defined $self->get_property('blackholeip6') ) {
		$self->set_property('blackholeip6' => '::1' );
	}
	# init adblock 
	try {
		Dnsproxy::Adblock::initadblock ({ blacklist => $self->get_property('blacklistfile'),
													whitelist => $self->get_property('whitelistfile'),
		}) ;
	} catch { croak $_ ; };

	# init resolver 
	try {
		Dnsproxy::Resolver::initresolver ({
			nameservers => $self->get_property('nameservers'),
			serchlist => $self->get_property('searchlist'),
			log_level => $self->get_property('log_level'),
			proto => $self->get_property('proto'),
			port => $self->get_property('resolverport'),
		}) ;
	} catch { croak $_ ; } ;
	return ;
}

1;
__END__
